from mongokit import Document

class Paper(Document):
  __database__ = 'predsynth'
  __collection__ = 'papers'
  structure = {
    'extracted_data': [ #input paragraphs
      [ #extracted sentences
        [ #operation data
          {
            'id': int,
            'sent_id': int,
            'operation': unicode,
            'operation_category': unicode,
            'entities': [unicode],
            'conditions': [unicode]
            # 'amounts': [unicode],
            # 'apparatuses': [unicode]
          }
        ]
      ]
    ],
    # 'operation_flows': [
    #   {
    #     'flow_id': int,
    #     'flows_from': int,
    #     'flows_to': int,
    #     'entity_set': [Entity],
    #     'operations': [unicode],
    #     'actions': [unicode],
    #     'entities':[[Entity]],
    #     'conditions': [[tuple]],
    #     'amounts': [[tuple]],
    #     'apparatuses': [[unicode]]
    #   }
    # ]
    #'doi': str,
    #'pii': str,
    #'file_loc': str,
    'title': unicode,
    'abstract': unicode,
    #'searched_from': [unicode],
    'target_materials': [unicode],
    'tagged_xml': unicode,
    'paragraphs': [unicode],
    'para_features': [
      [int]
    ],
    'is_para_methods': [int]
  }

  def get_ext_data(self, data_type):
    data = []
    for inp_para in self['extracted_data']:
      for ext_sentence in inp_para:
        for op_data in ext_sentence:
          if isinstance(op_data[data_type], list):
            if len(op_data[data_type]) > 0: data.extend([str(datum) for datum in op_data[data_type]])
          else:
            data.append(str(op_data[data_type]))

    return data

  def sort_all_operations(self):
    for i, inp_para in enumerate(self['extracted_data']):
      for j, ext_sentence in enumerate(inp_para):
        self['extracted_data'][i][j] = sorted(ext_sentence, key=lambda op_data: int(op_data['sent_id']))


class Entity(Document):
  __database__ = 'predsynth'
  __collection__ = 'entities'
  structure = {
    'is_chemical' : bool, #Is it a chemical or something more 'generic'?
    'MP_ID' : str, #Materials Project ID
    'ICSD_ID': str, #ICSD ID
    'aliases': [unicode], #names and all aliases
    'paper_ids': [unicode] #IDs of papers which contain this entity
  }

  def generate_common_name_from_formula(self):
    pass

  def get_MPID(self):
    return self.MP_ID

  def get_all_aliases(self):
    return self.aliases

  def get_all_paper_ids(self):
    return self.paper_ids


class SynthTrainParagraph(Document):
  __database__ = 'predsynth'
  __collection__ = 'synth_train_paragraphs'

  structure = {
    'paragraph': unicode,
    'is_para_method': int
  }
